#!/usr/bin/perl
#
# This script removes punctuation from CoNLL 2006 and CoNLL 2007 data files.
#
# Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
# Copyright 2013 by Milan Straka <fox@ucw.cz>.
#
# This script is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this script.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use open qw/:std :utf8/;

my @token;
while (<>) {
    chomp;
    if (/^\s*$/) {
        # nothing can depend on punctuation
        foreach my $i (1 .. $#token) {
            next if @{$token[$i]} < 7;
            while ($token[$i][6] != 0 && is_punctuation($token[$token[$i][6]][1])) {
                $token[$i][6] = $token[$token[$i][6]][6];
            }
        }
        # get mapping
        my @mapping = (0);
        my $counter = 0;
        foreach my $i (1 .. $#token) {
            if (!is_punctuation($token[$i][1])) {
                $counter++;
            }
            push @mapping, $counter;
        }
        # remove punctuation
        foreach my $i (1 .. $#token) {
            if (!is_punctuation($token[$i][1])) {
                $token[$i][0] = $mapping[$token[$i][0]];
                $token[$i][6] = $mapping[$token[$i][6]] unless @{$token[$i]} < 7;
                print join("\t", @{$token[$i]}) . "\n";
            }
        }
        print "\n" if $counter;
        @token = ();
    } elsif (!/^\t/) {
        my @items = split /\t/;
        $token[$items[0]] = \@items;
    }
}

sub is_punctuation {
    my $form = shift;
    return 1 if ($form !~ /^[\w\d]/ || $form =~ /^_/);
    return 0;
}
