#!/usr/bin/perl
#
# This script computes the unlabled attachment score (UAS), unlabeled
# undirected attachment score (UUAS) and # neutral edge direction score
# (NED) given the test and reference CONLL files.
#
# Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
# Copyright 2013 by Milan Straka <fox@ucw.cz>.
#
# This script is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this script.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use open qw/:std :utf8/;

use Getopt::Long;

my @parent;
my $uas = 0;
my $uuas = 0;
my $ned = 0;
my $total = 0;

my $dump_sentences = 0;

GetOptions ('s|sentences=i' => \$dump_sentences);

open (GOLD, "<", $ARGV[0]) or die;
my $s = 0;
while (<GOLD>) {
    chomp;
    if (/^\d/) {
        my @item = split(/\t/, $_);
        $parent[$s][$item[0]] = $item[6];
    }
    else {
        $s++;
    }
}
close GOLD;
shift;

my $gold_sentences = $s;
my @sentences = ();

$s = 0;
while (<>) {
    chomp;
    if (/^\d/) {
        $total++;
        $sentences[$s] .= $_ . "\n" if $s < $dump_sentences;
        my @item = split(/\t/, $_);
        if ($item[6] == $parent[$s][$item[0]]) {
            $uas++;
            $uuas++;
            $ned++;
        }
        elsif ($item[6] && $item[0] == $parent[$s][$item[6]]) {
            $uuas++;
            $ned++;
        }
        elsif ($parent[$s][$item[0]] && $item[6] == $parent[$s][$parent[$s][$item[0]]]) {
            $ned++;
        }
    }
    else {
        $s++;
        last if $s >= $gold_sentences;
    }
}

printf "UAS: %.3f\tUUAS: %.3f\tNED: %.3f\n", ($uas*100/$total), ($uuas*100/$total), ($ned*100/$total);
print join "\n", (@sentences, "") if @sentences;
