// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

// Headers available in all sources
#include <cassert>
#include <iostream>
using namespace std;

// Foreach loop construct
#define foreach(var, range) for (auto _foreach_it = range.begin(); _foreach_it != range.end(); _foreach_it++) for(var = *_foreach_it; ; ({break;}))

// The type str with the std::string interface. It can be either
// directly the std::string, of g++ vstring can be used.
#ifdef USE_STL_STRING
  #include <string>
  typedef std::string str;
#else
  #include <ext/vstring.h>
  typedef __gnu_cxx::__sso_string str;

  // We need the hash<str> instance, which is not provided by g++-4.5 and former.
  #if __GNUC__ == 4 && __GNUC_MINOR__ <= 5 && !defined(__clang__)
  #include <functional>
  namespace std {
  template<> inline size_t hash<str>::operator()(str s) const {
      return _Fnv_hash<>::hash(s.data(), s.length());
    }
  }
  #endif
#endif

// Printf-like exception throwing function.
void runtime_errorf(const char* fmt, ...)
#ifdef __GNUC__
  __attribute__ ((noreturn))
#endif
  ;

// Getline ignoring \r.
istream& gettextline(istream& is, str& line);
