// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <unordered_map>
#include <tuple>
#include <vector>

// Corpus can have
// * at most 2^15-1 nodes
// * at most 2^15-1 tags
// * at most 2^31-1 forms
// * at most 2^31-1 lowercased forms

class sentence;

enum { LEFT = 0, RGHT = 1, DIRS = 2 };

class node {
 public:
  int form, lcform;
  short ord;
  short tag, oritag;
  short parent;
  short sibling[DIRS];
  short children[DIRS];
  short outer_child[DIRS];
  short range[DIRS];

  node(short ord, int form, int lcform, short tag, short oritag) :
    form(form), lcform(lcform), ord(ord), tag(tag), oritag(oritag), parent(-1), sibling{-1,-1}, children(), outer_child{-1,-1}, range{ord, ord} {}
};

class sentence {
 public:
  int ord;
  vector<node> nodes;
  int root;
  int comp;

  sentence(int ord) : ord(ord), root(-1), comp(0) {}

  void add_node(int form, int lcform, int tag, int oritag);
  void flatten();
  void change_parent(node& n, int parent);

  void assert_valid();

 private:
  void assert_valid(const node& n);
};

class corpus {
 public:
  vector<str> forms, lcforms, tags;
  unordered_map<str, int> forms_map, lcforms_map, tags_map;
  vector<sentence> sentences;

  void load_tagmap(istream& in);
  void load(istream& in, int form_column, int tag_column);
  void save(ostream& out);

  void assert_valid();

 private:
  unordered_map<str, str> tagmap;
  int form_id(const str& form);
  int lcform_id(const str& form);
  int tag_id(const str& tag);
};

