// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <memory>

#include "corpus/corpus.h"
#include "gibbs_sampler_block.h"
#include "gibbs_sampler_incremental.h"
#include "models/model.h"

class gibbs_sampler {
 public:
  gibbs_sampler(corpus& c, bool verbose) : c(c), verbose(verbose) {}
  ~gibbs_sampler();

  enum sampling_mode { INCREMENTAL, BLOCK_MAX, BLOCK_MAX_NOUPDATE, BLOCK_RND, BLOCK_RND_NOUPDATE };
  enum decoding_mode { MST, BEST_PROB };

  void add_model(model* m);
  void initialize();
  void sample(sampling_mode mode, int iterations, int samples);
  void decode(decoding_mode mode);

 private:
  corpus& c;
  bool verbose;
  vector<model*> models;
  gibbs_sampler_block block_sampler;
  gibbs_sampler_incremental incremental_sampler;

  enum { incremental_progress_step = 1<<13, block_progress_step = 1<<10 };
  enum { initialize_max_children = 100 };

  void initialize_sentence(sentence& s, int left, int right);
  double initialize_models(bool compute_logprob = true);

  // Edge sampling for MST decoding
  struct sentence_info {
    // Counts of node_ord shifted by 1, 0 is the technical root.
    vector<vector<int>> counts;

    sentence_info(int nodes) { for (int i = 0; i <= nodes; i++) counts.emplace_back(nodes + 1, 0); }
  };

  vector<sentence_info> sampled;
};
