// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <vector>

#include "common.h"
#include "models/model.h"

class gibbs_sampler_block {
 public:
  void construct(const vector<model*>& models, bool verbose);
  void sample(sentence& s, bool best, bool update_counts);

  enum { MAX_LEN = 60, OVERLAP = 20 };
 private:
  vector<block_model*> block_models;

  vector<block_model*> edge_models;
  vector<block_model*> range_models;
  vector<block_model*> stop_models;

  typedef double prob_t; // Type of probability, either double or long double.
  // Most important probabilities which are computed using dynamic programming:
  prob_t p[MAX_LEN][MAX_LEN][MAX_LEN][2]; // The p[g][l][r][s] is probability of one tree with parent g
                                          // and range <l..r>. The tree is assumed to have some (s=0)
                                          // or no (s=1) further siblings.
  prob_t q[MAX_LEN][MAX_LEN][MAX_LEN];    // The q[g][l][r] is probability of one or more trees with parent g
                                          // and range <l..r>. Trees are assumed to have no further siblings.

  // Buffered probabilities of all models:
  prob_t _edge_probs[1][MAX_LEN];         // This is here to allow -1 as a governor in the following edge_probs.
  prob_t edge_probs[MAX_LEN][MAX_LEN];    // The governor can be -1.
  prob_t range_probs[MAX_LEN][MAX_LEN];
  prob_t stop_adj_probs[2][MAX_LEN][2];
  prob_t stop_fri_probs[2][MAX_LEN][MAX_LEN];

  vector<int> parents;

  // High-order sampling
  template <class Combine, class Choose>
  void sample(sentence& s);

  template <class Combine>
  void count_probs(sentence& s, int from, int nodes);

  template <class Combine>
  prob_t tree_prob(int l, int r);

  template <class Choose>
  void choose_root(int parent, int l, int r);

  template <class Choose>
  void choose_left_children(int parent, int l, int r);

  template <class Choose>
  void choose_rght_children(int parent, int l, int r);

  class Combine_max { public: static inline prob_t apply(prob_t a, prob_t b); };
  class Choose_max { public: static inline int apply(const vector<prob_t>& probs); };
  class Combine_rnd { public: static inline prob_t apply(prob_t a, prob_t b); };
  class Choose_rnd { public: static inline int apply(const vector<prob_t>& probs); };
};
