// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <vector>

#include "common.h"
#include "models/model.h"

class gibbs_sampler_incremental {
 public:
  void construct(const vector<model*>& models, bool verbose);
  void sample(sentence& s, int iter);

 private:
  class sampling_space {
   public:
    struct sample_type {
      unsigned dir : 1;
      unsigned ch_outer : 15;
      unsigned ch_inner : 15;
      unsigned comp : 1;
    };

    void construct(const vector<model*>& models, bool verbose);
    inline sample_type init(sentence& s, node& n);
    inline bool next(sample_type& sample, double& prob);
    inline void choose(sample_type target);
    inline void change_parent(int n, int new_parent, double* prob = NULL);

    vector<incremental_model*> incremental_models;
    vector<incremental_comp_sampling_model*> incremental_comp_sampling_models;

    sentence* s;
    sample_type initial;
    int n_lower, n_upper;
    enum { OUTER = 0, INNER = 1 };
    int ch_lower[2], ch_upper[2];
    double checkpoint_prob;
  };

  vector<int> perm;
  sampling_space sampling_space;
  vector<pair<sampling_space::sample_type, double>> samples;
};
