// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <vector>
#include <unordered_map>

#include "model.h"

class attach_model : public incremental_model, public incremental_comp_sampling_model, public block_model {
 public:
  attach_model(const model_options& options);

  virtual void construct(const corpus& c);
  virtual void initialize(const corpus& c, double* logprob = NULL);

  virtual void change_parent(const sentence& s, const node& n, int new_parent, double* prob = NULL);
  virtual incremental_comp_sampling_model* comp_sampling();
  virtual void change_comp(const sentence& s, double* prob = NULL);

  virtual bool use_edge_prob() { return true; }
  virtual double edge_prob(const sentence& s, int parent, int node);
  virtual void count_sentence(const sentence& s, int delta);
 private:
  double alpha;
  bool use_cross;
  bool use_comp;
  bool use_fg, use_tg;
  bool unigram_base;
  bool use_p0;
  double beta;
  vector<vector<int>> nearest_punctuation;

  int tags;
  int forms;
  
  vector<double> tag_prob;

  inline void init_counts();
  inline void count_edge(int delta, const sentence& s, const node& n, const node* p, double* prob, int swap_comp = 0);
  inline void count_sentence(const sentence& s, int delta, double* logprob);

  struct counts_info {
    int count;
    vector<int> children;

    counts_info() : count(), children(tags) {}
    void operator+=(const counts_info& other) { count += other.count; for (int i = 0; i < tags; i++) children[i] += other.children[i]; }
    static int tags;
  };
  vector<counts_info> counts;
  unordered_map<int, counts_info> big_counts;

  str formprobs_file;
  vector<double> formprobs;

  // Support for sampling model state
  virtual void sample_state();
  virtual void use_sampled_state();
  vector<counts_info> sampled_counts;
  unordered_map<int, counts_info> sampled_big_counts;
};
