// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <unordered_map>

#include "corpus/corpus.h"

class incremental_model;
class incremental_comp_sampling_model;
class block_model;

class model {
 public:
  virtual ~model() {}
  virtual void construct(const corpus& c) = 0;
  virtual void initialize(const corpus& c, double* logprob = NULL) = 0;
  virtual void sample_state() {}
  virtual void use_sampled_state() {}

  virtual incremental_model* incremental() { return NULL; }
  virtual block_model* block() { return NULL; }
};

class incremental_model : public virtual model {
 public:
  virtual void change_parent(const sentence& s, const node& n, int new_parent, double* prob = NULL) = 0;

  virtual incremental_model* incremental() { return this; }
  virtual incremental_comp_sampling_model* comp_sampling() { return NULL; }
};

class incremental_comp_sampling_model {
 public:
  virtual void change_comp(const sentence& s, double* prob = NULL) = 0;
};

class block_model : public virtual model {
 public:
  virtual bool use_edge_prob() { return false; }
  virtual double edge_prob(const sentence& /*s*/, int /*parent*/, int /*node*/) { return 1; }

  virtual bool use_range_prob() { return false; }
  virtual double range_prob(const sentence& /*s*/, int /*range_left*/, int /*range_rght*/) { return 1; }

  virtual bool use_stop_prob() { return false; }
  virtual double stop_prob(const sentence& /*s*/, bool /*stop*/, int /*parent*/, int /*dir*/, int /*fringe*/) { return 1; }

  virtual void count_sentence(const sentence& /*s*/, int /*delta*/) {}

  virtual block_model* block() { return this; }
};

////////// Model helper functions

// Options
typedef unordered_map<str, str> model_options;
model_options parse_model_options(const str& options);

// Logprob bounds
#define logprob_min_bound 1e-200
#define logprob_max_bound 1e200

// Multiplicative increment
#define multiplicative_increment(prob, incr, delta) (((delta) >= 0) ? prob *= incr : prob /= incr)
