// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <vector>

#include "model.h"

class stop_model : public incremental_model, public incremental_comp_sampling_model, public block_model {
 public:
  stop_model(const model_options& options);

  virtual void construct(const corpus& c);
  virtual void initialize(const corpus& c, double* logprob = NULL);

  virtual void change_parent(const sentence& s, const node& n, int new_parent, double* prob = NULL);
  virtual incremental_comp_sampling_model* comp_sampling();
  virtual void change_comp(const sentence& s, double* prob = NULL);

  virtual bool use_stop_prob() { return true; }
  virtual double stop_prob(const sentence& s, bool stop, int parent, int dir, int fringe);
  virtual void count_sentence(const sentence& s, int delta);
 private:
  double alpha;
  double mult1;
  double mult0;
  double base1;
  double base0;
  bool fromfile1;
  bool fromfile0;
  bool use_tf, use_tg;
  bool use_comp;
  str priors_file;

  int tags;

  struct priors_info {
    double prob1;
    double prob0;
    bool specified;

    priors_info() : prob1(1), prob0(1), specified(false) {}
  };
  vector<priors_info> priors;


  inline void init_counts();
  inline void count_edge(int delta, const sentence& s, bool stop, int parent, int dir, int fringe, double* prob, int swap_comp = 0);
  inline void count_sentence(const sentence& s, int delta, double* logprob = NULL);

  struct counts_info {
    int count;
    int stop[2];

    counts_info() : count(), stop() {}
    void operator+=(const counts_info& other) { count += other.count; stop[0] += other.stop[0]; stop[1] += other.stop[1]; }
  };
  vector<counts_info> counts;

  // Support for sampling model state
  virtual void sample_state();
  virtual void use_sampled_state();
  vector<counts_info> sampled_counts;
};
