// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#include <chrono>

#include "common.h"
#include "distribution.h"

int distribution::uniform(int min, int max) {
  unsigned range = max - min;
  unsigned r = instance.g();

  // At first try coarse bound, which can be computed fast and works almost every time.
  unsigned bound = instance.g.max() - range;
  if (r > bound) {
    unsigned bound = instance.g.max() - instance.g.max() % range;
    while (r >= bound) r = instance.g();
  }

  return min + r % range;
}

double distribution::real(double min, double max) {
  return min + instance.g() * (max - min) / (double(instance.g.max()) + 1);
}

void distribution::permutation(int min, int max, vector<int>& perm) {
  int N = max - min;
  perm.resize(N);
  for (int i = 0; i < N; i++) perm[i] = i + min;

  for (int i = N-1; i > 0; i--) {
    int j = uniform(0, i + 1);
    if (j < i) {
      int tmp = perm[j];
      perm[j] = perm[i];
      perm[i] = tmp;
    }
  }
}

distribution::distribution() : g(chrono::high_resolution_clock::now().time_since_epoch().count()) {}
//distribution::distribution() : g(1234) {}
distribution distribution::instance;
