// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#include <cstring>

#include <getopt.h>

#include "common.h"
#include "options.h"

int parse_options(int argc, char* argv[], unordered_map<str, str>& options) {
  struct option opts[options.size()+1]; memset(opts, 0, sizeof(opts));
  bool used[options.size()]; memset(used, 0, sizeof(used));

  int opts_len = 0;
  foreach (auto& option, options) {
    opts[opts_len].name = option.first.c_str();
    opts[opts_len++].has_arg = optional_argument;
  }

  int longid, ret;
  while (ret = getopt_long_only(argc, argv, "", opts, &longid), ret != -1) {
    if (ret != 0) exit(1);

    options[opts[longid].name] = optarg ? optarg : "";
    used[longid] = true;
  }

  for (int i = 0; i < opts_len; i++)
    if (!used[i] && options[opts[i].name].empty())
      options.erase(opts[i].name);

  return optind;
}
