// This file is part of udpc.
//
// Copyright 2013 by Milan Straka <fox@ucw.cz>.
// Copyright 2013 by David Marecek <marecek@ufal.mff.cuni.cz>.
//
// Udpc is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Udpc is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with udpc.  If not, see <http://www.gnu.org/licenses/>.

#include <cwctype>

#include "common.h"
#include "utils.h"

void split(const str &text, char sep, vector<str>& tokens) {
  tokens.clear();

  unsigned long start = 0, end = 0;
  while ((end = text.find(sep, start)) != str::npos) {
    tokens.push_back(text.substr(start, end - start));
    start = end + 1;
  }
  tokens.push_back(text.substr(start));
}

void uppercase(const str& in, str& out) {
  vector<wchar_t> wchars;

  to_wchars(in, wchars);
  foreach (auto& w, wchars) w = towupper(w);
  from_wchars(wchars, out);
}

void lowercase(const str& in, str& out) {
  vector<wchar_t> wchars;

  to_wchars(in, wchars);
  foreach (auto& w, wchars) w = towlower(w);
  from_wchars(wchars, out);
}

void to_wchars(const str& in, vector<wchar_t>& out) {
  // 1) convert to wchar_t[]
  out.resize(in.size() + 1);

  mbstate_t mb = mbstate_t();
  const char* in_left = in.c_str();
  size_t len = mbsrtowcs(out.data(), &in_left, out.size(), &mb);
  if (len == (size_t) -1 || in_left)
    runtime_errorf("Cannot convert %s to wide character representation", in.c_str());

  out.resize(len + 1);
}

void from_wchars(const vector<wchar_t>& in, str& out) {
  mbstate_t mb = mbstate_t();
  const wchar_t* wchars_left = in.data();

  size_t len = wcsrtombs(NULL, &wchars_left, 0, &mb);
  if (len == (size_t) -1)
    runtime_errorf("Cannot convert string beginning with %d from wide character representation", in[0]);

  out.resize(len);
  len = wcsrtombs((char*) out.c_str(), &(wchars_left = in.data()), len + 1, &mb);
  if (len == (size_t) -1 || len != out.size() || wchars_left)
    runtime_errorf("Cannot convert string beginning with %d from wide character representation", in[0]);
}
